@@ -1,4 +1,5 @@
 #
+# Copyright (c) 2005 Canonical
 # Copyright (c) 2004 Conectiva, Inc.
 #
 # Written by Gustavo Niemeyer <niemeyer@conectiva.com>
@@ -68,6 +69,10 @@ else:
             return res
         __builtins__["raw_input"] = raw_input
 
+# Just install it
+from smart.util import protectedread
+del protectedread
+
 class Proxy:
     def __init__(self, object=None):
         self.object = object
@@ -120,6 +125,20 @@ def init(command=None, argv=None,
         ifacename = "text"
     iface.object = createInterface(ifacename, ctrl, command, argv)
 
+    # Python's default to SIGPIPE is SIG_IGN, set by initsigs() in
+    # pythonrun.c. As a side effect:
+    #
+    #   >>> os.system("yes | read any")
+    #   yes: standard output: Broken pipe
+    #   yes: write error
+    #
+    #   >>> signal.signal(signal.SIGPIPE, signal.SIG_DFL)
+    #   >>> os.system("yes | read any")
+    #   >>>
+    #
+    import signal
+    signal.signal(signal.SIGPIPE, signal.SIG_DFL)
+
     return ctrl
 
 def initDistro(ctrl):
