@@ -1,4 +1,5 @@
 #
+# Copyright (c) 2005 Canonical
 # Copyright (c) 2004 Conectiva, Inc.
 #
 # Written by Gustavo Niemeyer <niemeyer@conectiva.com>
@@ -124,8 +125,12 @@ class RPMHeaderPackageInfo(PackageInfo):
     def getPathList(self):
         if self._path is None:
             paths = self._h[rpm.RPMTAG_OLDFILENAMES]
+            if type(paths) != list:
+                paths = [paths]
             modes = self._h[rpm.RPMTAG_FILEMODES]
             if modes:
+                if type(modes) != list:
+                    modes = [modes]
                 self._path = {}
                 for i in range(len(paths)):
                     self._path[paths[i]] = modes[i]
@@ -227,6 +232,8 @@ class RPMHeaderLoader(Loader):
             if n:
                 f = h[1048] # RPMTAG_REQUIREFLAGS
                 v = h[1050] # RPMTAG_REQUIREVERSION
+                if type(f) != list:
+                    f = [f]
                 reqdict = {}
                 for i in range(len(n)):
                     ni = n[i]
@@ -257,7 +264,11 @@ class RPMHeaderLoader(Loader):
                     vi = v[i] or None
                     if vi and vi[:2] == "0:":
                         vi = vi[2:]
-                    cnfargs.append((Cnf, n[i], CM.get(f[i]&CF), vi))
+                    if i == 0 and type(f) != list:
+                        fi = f
+                    else:
+                        fi = f[i]
+                    cnfargs.append((Cnf, n[i], CM.get(fi&CF), vi))
             else:
                 cnfargs = []
 
@@ -272,7 +283,11 @@ class RPMHeaderLoader(Loader):
                     vi = v[i] or None
                     if vi and vi[:2] == "0:":
                         vi = vi[2:]
-                    upgargs.append((Obs, n[i], CM.get(f[i]&CF), vi))
+                    if i == 0 and type(f) != list:
+                        fi = f
+                    else:
+                        fi = f[i]
+                    upgargs.append((Obs, n[i], CM.get(fi&CF), vi))
                 cnfargs.extend(upgargs)
                 upgargs.append(obstup)
             else:
@@ -307,6 +322,8 @@ class RPMHeaderLoader(Loader):
             if searcher.path:
                 paths = h[rpm.RPMTAG_OLDFILENAMES]
                 if paths:
+                    if type(paths) != list:
+                        paths = [paths]
                     for spath, cutoff in searcher.path:
                         for path in paths:
                             _, newratio = globdistance(spath, path, cutoff, ic)
@@ -381,9 +398,9 @@ class RPMHeaderListLoader(RPMHeaderLoader):
                                 "As a consequence, Smart will consume "
                                 "extra memory."))
 
-            self.__class__.getHeaders = self.getHeadersHDL
-            self.__class__.getHeader = self.getHeaderHDL
-            self.__class__.loadFileProvides = self.loadFileProvidesHDL
+            self.__class__.getHeaders = self.getHeadersHDL.im_func
+            self.__class__.getHeader = self.getHeaderHDL.im_func
+            self.__class__.loadFileProvides = self.loadFileProvidesHDL.im_func
 
             self._hdl = rpm.readHeaderListFromFile(self._filename)
 
@@ -454,7 +471,10 @@ class RPMHeaderListLoader(RPMHeaderLoader):
         h, offset = rpm.readHeaderFromFD(file.fileno())
         bfp = self.buildFileProvides
         while h:
-            for fn in h[1027]: # RPMTAG_OLDFILENAMES
+            fnlst = h[1027] # RPMTAG_OLDFILENAMES
+            if type(fnlst) != list:
+                fnlst = [fnlst]
+            for fn in fnlst:
                 fn = fndict.get(fn)
                 if fn and offset in self._offsets:
                     bfp(self._offsets[offset], (RPMProvides, fn, None))
@@ -464,7 +484,10 @@ class RPMHeaderListLoader(RPMHeaderLoader):
     def loadFileProvidesHDL(self, fndict):
         bfp = self.buildFileProvides
         for offset, h in enumerate(self._hdl):
-            for fn in h[1027]: # RPMTAG_OLDFILENAMES
+            fnlst = h[1027] # RPMTAG_OLDFILENAMES
+            if type(fnlst) != list:
+                fnlst = [fnlst]
+            for fn in fnlst:
                 fn = fndict.get(fn)
                 if fn and offset in self._offsets:
                     bfp(self._offsets[offset], (RPMProvides, fn, None))
@@ -653,7 +676,10 @@ class RPMDirLoader(RPMHeaderLoader):
                 iface.error("%s: %s" % (os.path.basename(filepath), e))
             else:
                 file.close()
-                for fn in h[1027]: # RPMTAG_OLDFILENAMES
+                fnlst = h[1027] # RPMTAG_OLDFILENAMES
+                if type(fnlst) != list:
+                    fnlst = [fnlst]
+                for fn in fnlst:
                     fn = fndict.get(fn)
                     if fn:
                         bfp(self._offsets[i], (RPMProvides, fn, None))
