@@ -1,4 +1,5 @@
 #
+# Copyright (c) 2005 Canonical
 # Copyright (c) 2004 Conectiva, Inc.
 #
 # Written by Gustavo Niemeyer <niemeyer@conectiva.com>
@@ -20,17 +21,19 @@
 # Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 #
 from smart.backends.rpm.rpmver import splitarch
-from smart.backends.rpm.base import rpm, getTS
 from smart.util.filetools import setCloseOnExec
 from smart.sorter import ChangeSetSorter, LoopError
 from smart.const import INSTALL, REMOVE, BLOCKSIZE
 from smart.pm import PackageManager
 from smart import *
+import tempfile
 import sys, os
 import codecs
 import locale
+import thread
 import errno
 import fcntl
+import time
 
 ENCODING = locale.getpreferredencoding()
 
@@ -94,8 +97,20 @@ class RPMPackageManager(PackageManager):
             flags |= rpm.RPMTRANS_FLAG_TEST
         ts.setFlags(flags)
 
-        if not sysconf.get("rpm-check-signatures", False):
-            ts.setVSFlags(rpm._RPMVSF_NOSIGNATURES)
+        # Set rpm verbosity level.
+        levelname = sysconf.get('rpm-log-level')
+        level = {
+            'emerg':   rpm.RPMLOG_EMERG,
+            'alert':   rpm.RPMLOG_ALERT,
+            'crit':    rpm.RPMLOG_CRIT,
+            'err':     rpm.RPMLOG_ERR,
+            'warning': rpm.RPMLOG_WARNING,
+            'notice':  rpm.RPMLOG_NOTICE,
+            'info':    rpm.RPMLOG_INFO,
+            'debug':   rpm.RPMLOG_DEBUG
+        }.get(levelname)
+        if level is not None:
+            rpm.setVerbosity(level)
 
         # Let's help RPM, since it doesn't do a good
         # ordering job on erasures.
@@ -182,6 +197,7 @@ class RPMPackageManager(PackageManager):
         if force or reinstall:
             probfilter |= rpm.RPMPROB_FILTER_REPLACEPKG
             probfilter |= rpm.RPMPROB_FILTER_REPLACEOLDFILES
+            probfilter |= rpm.RPMPROB_FILTER_REPLACENEWFILES
         ts.setProbFilter(probfilter)
         cb = RPMCallback(prog, upgradednames)
         cb.grabOutput(True)
@@ -189,6 +205,7 @@ class RPMPackageManager(PackageManager):
         try:
             probs = ts.run(cb, None)
         finally:
+            del getTS.ts
             cb.grabOutput(False)
             prog.setDone()
             if probs:
@@ -202,6 +219,7 @@ class RPMCallback:
         self.data = {"item-number": 0}
         self.fd = None
         self.rpmout = None
+        self.rpmoutbuffer = ""
         self.lasttopic = None
         self.topic = None
 
@@ -215,18 +233,15 @@ class RPMCallback:
                 reader = codecs.getreader(ENCODING)
                 sys.stdout = writer(os.fdopen(os.dup(1), "w"))
                 sys.stderr = writer(os.fdopen(os.dup(2), "w"))
-                pipe = os.pipe()
-                os.dup2(pipe[1], 1)
-                os.dup2(pipe[1], 2)
-                os.close(pipe[1])
-                self.rpmout = reader(os.fdopen(pipe[0], "r"))
-                setCloseOnExec(self.rpmout.fileno())
-                flags = fcntl.fcntl(self.rpmout.fileno(), fcntl.F_GETFL, 0)
-                flags |= os.O_NONBLOCK
-                fcntl.fcntl(self.rpmout.fileno(), fcntl.F_SETFL, flags)
+                fd, rpmoutpath = tempfile.mkstemp("-smart-rpm-out.txt")
+                os.dup2(fd, 1)
+                os.dup2(fd, 2)
+                os.close(fd)
+                self.rpmout = reader(open(rpmoutpath))
+                os.unlink(rpmoutpath)
         else:
             if self.rpmout:
-                self._rpmout()
+                self._process_rpmout()
                 os.dup2(sys.stdout.fileno(), 1)
                 os.dup2(sys.stderr.fileno(), 2)
                 sys.stdout = self.stdout
@@ -235,16 +250,17 @@ class RPMCallback:
                 del self.stderr
                 self.rpmout.close()
                 self.rpmout = None
+                self.rpmoutbuffer = ""
 
-    def _rpmout(self):
+    def _process_rpmout(self, tobuffer=False):
         if self.rpmout:
-            try:
-                output = self.rpmout.read(BLOCKSIZE)
-            except (OSError, IOError), e:
-                if e[0] != errno.EWOULDBLOCK:
-                    raise
-            else:
-                if output:
+            output = self.rpmout.read()
+            if output or not tobuffer and self.rpmoutbuffer:
+                if tobuffer:
+                    self.rpmoutbuffer += output
+                else:
+                    output = self.rpmoutbuffer+output
+                    self.rpmoutbuffer = ""
                     if self.topic and self.topic != self.lasttopic:
                         self.lasttopic = self.topic
                         iface.info(self.topic)
@@ -252,7 +268,8 @@ class RPMCallback:
 
     def __call__(self, what, amount, total, infopath, data):
 
-        self._rpmout()
+        if self.rpmout:
+            self._process_rpmout()
 
         if what == rpm.RPMCALLBACK_INST_OPEN_FILE:
             info, path = infopath
@@ -321,4 +338,6 @@ class RPMCallback:
                 self.prog.setSubDone(subkey)
             self.prog.show()
 
+from smart.backends.rpm.base import rpm, getTS
+
 # vim:ts=4:sw=4:et
