@@ -1,4 +1,5 @@
 #
+# Copyright (c) 2005 Canonical
 # Copyright (c) 2004 Conectiva, Inc.
 #
 # Written by Gustavo Niemeyer <niemeyer@conectiva.com>
@@ -847,20 +848,22 @@ class LocalMediaHandler(FileHandler):
         mediaset = self._fetcher.getMediaSet()
         for i in range(len(self._queue)-1,-1,-1):
             item = self._queue[i]
-            itempath = item.getURL().path
-            media = item.getInfo("media")
-            if not media:
-                media = mediaset.getDefault()
-                if media:
-                    media.mount()
-                else:
-                    mediaset.mountAll()
-                    media = mediaset.findFile(itempath)
-                if not media or not media.isMounted():
-                    item.setFailed(_("Media not found"))
-                    del self._queue[i]
-                    continue
-            item.setURL(media.joinURL(itempath))
+            url = item.getURL()
+            if url.scheme == "localmedia":
+                itempath = url.path
+                media = item.getInfo("media")
+                if not media:
+                    media = mediaset.getDefault()
+                    if media:
+                        media.mount()
+                    else:
+                        mediaset.mountAll()
+                        media = mediaset.findFile(itempath)
+                    if not media or not media.isMounted():
+                        item.setFailed(_("Media not found"))
+                        del self._queue[i]
+                        continue
+                item.setURL(media.joinURL(itempath))
 
 Fetcher.setHandler("localmedia", LocalMediaHandler, local=True)
 
@@ -920,7 +923,8 @@ class FTPHandler(FetcherHandler):
             ftp.connect(url.host, url.port)
             ftp.login(url.user, url.passwd)
         except (socket.error, ftplib.Error, EOFError), e:
-            if isinstance(e, ftplib.error_perm) and active:
+            if (isinstance(e, ftplib.error_perm) or
+                isinstance(e, ftplib.error_temp)) and active:
                 item.reset()
                 self._lock.acquire()
                 self._queue.append(item)
@@ -1646,7 +1650,7 @@ try:
 except ImportError:
     pass
 else:
-    schemes = pycurl.version_info()[-1]
+    schemes = pycurl.version_info()[8]
     for scheme in schemes:
         if scheme != "file":
             Fetcher.setHandler(scheme, PyCurlHandler)
